options nosyntaxcheck;

title1 'Contin Time Causal Mediation Analysis - GenLogis - 2 step';

****************DENTAL DATA VARIABLE DEFINITIONS*******************************;
*  variables:
*    exposure (X):
*       SES: 1 if high, 0 if low
*    baseline (C):
*       bthgrp: 1 if very low birth weight (VLBW), 0 if normal birth weight (NBW),
*       sex: 1 = male, 0 = female,
*    mediator (M):
*       sdrk8, sdrk18, sdrk36
*       (sugary drinks use score at 8, 18, 36 mths)
*    final outcome (Y):
*       dmft8, dmft18, dmft36
*       (number of decayed, missing, filled teeth at 8, 18, 36 mths)
*******************************************************************************;

/******* Example of pre-processing with dental data ******/

data ECC;
   set suchi2.dentdat;
run;

/*    Macro to obtain desired cases from original dental data set  
      options are COMPLETE (complete cases) and INCOMPLETE (using
      all subjects with at least 1 Y and at least 1 M). Assumes
      baseline variables (including the exposure variable are complete)  */

%MACRO dataset(case);
  %IF &case = complete %THEN %DO;
  data ECCM;
	* Produces data set with 195 subjects;
	set ECC;
	 if (sdrk8=. or sdrk18=. or sdrk36=.) then delete;   
	 if (dmft8=. or dmft18=. or dmft36=.) then delete;   

	 if ses=. then delete;                              
	 if bthgrp=. then delete;                            
	 if sex=. then delete;                               
  run;
  %END;

  %ELSE %IF &case = incomplete %THEN %DO;
  data ECCM;
	* Produces data set with 440 subjects;
	set ECC;
	 if (sdrk8=. and sdrk18=. and sdrk36=.) then delete; 
	 if (dmft8=. and dmft18=. and dmft36=.) then delete; 

	 if ses=. then delete;                              
	 if bthgrp=. then delete;                            
	 if sex=. then delete;                               
  run;
  %END;

%MEND dataset;
RUN;

%dataset(incomplete)
RUN;

/*  Inputs for Macro dataset:
1) case = complete (for complete cases) 
		= incomplete (for case with at least 1 M and at least 1 Y)
*/


/***********************************************************************************************************     
					Macro CMED - Version 1.1, updated 3/19/2017

The SAS macro CMED conducts a continuous time mediation analysis based on the method proposed
in Albert et al., 2019, Continuous Time Causal Mediation Analysis, Statistics in Medicine.
The method is applicable when the final outcome (Y) and mediator (M) are measured repeatedly over time.
The approach asssumes that Y and M follow generalized logistic models and are distributed as negative binomial
(though other distribution may be allowed in future versions).

The output provides estimates of the natural direct and indirect effects at user-specified times. Also 
computed is the area between the curves (ABC) as an overall measure of the indirect effect. Effects may 
be defined on a mean difference or mean ratio scale.

The macro also allows computation of an ABC for a partial indirect (mediation) effect, corresponding to
a treatment starting (or exposure ending) at specified time t1. 

The methods uses a continuous time version of the mediation formula, and assumes a continuous time 
extension of the sequential ignorability assumption. Percentile method bootstrap 95% confidence intervals 
are computed for each effect. A plot of the predicted mean for Y as a function of time for different
interventions/exposures is also provided.

Note that the computational time may be long, particularly with a large number of bootstraps (and large
dataset).

***********************************************************************************************************/

%macro CMED (nboot, seed, randseed,
             yvar, nt, yt1, yt2, yt3, yt4, yt5, ts1, ts2, ts3, ts4, ts5,
             mvar, mt1, mt2, mt3, mt4, mt5,
             xvar, 
             cvar, nc, cvar1, cvar2, cvar3, cvar4, cvar5,
             tgd1, tgd2, t1, mtim, delg, mult,
             xstart, xend, interval,
             na, ta1, ta2, ta3, ta4, ta5,
             scale, datset);      
 
data _NULL_;                         /* count number of subjects in datset */
 if 0 then set &datset nobs=n;
 call symputx('nsamp',n);
 stop;
run;
%put no. of observations = &nsamp;

data dat2 (keep = id X C1-C&nc M1-M&nt Y1-Y&nt btsamp);
	set &datset;
	array M[*] M1-M&nt;
	array Y[*] Y1-Y&nt;
	array C[*] C1-C&nc;
    %do j = 1 %to &nt;
        M[&j] = &&mt&j;
    %end;
    %do j = 1 %to &nt;
        Y[&j] = &&yt&j;
    %end;
	%if &nc > 0 %then %do;
       %do j = 1 %to &nc;
          C[&j] = &&cvar&j;
       %end;
    %end; 
    X = &xvar;
	btsamp = 1;  /* value 1 indicates original dataset */
run;

proc surveyselect data=dat2 out=Boot1 samprate=1 rep=&nboot method=urs seed=&seed
	outhits noprint;   
run;

data Boot;
	set Boot1;
	btsamp = replicate + 1;
run;

* Merge bootstrap and original data;

data btdat1;
	set dat2 Boot;
run;

proc sort data=btdat1;
	by btsamp id;
run;

data btdat2;
	set btdat1;
	idb=_N_ - (&nsamp * (btsamp - 1));  * provides unique ids for each bootstrap sample;
run; 

* Rearrange data structure to obtain one line per visit;

data repdat0 (keep = id idb X t M Y btsamp C1-C&nc); 
   array yy{&nt} Y1-Y&nt;
   array mm{&nt} M1-M&nt;
   array ts[*] ts1-ts&nt;
   set btdat2;
   by btsamp idb;
   %do i=1 %to &nt;
      Y = yy[&i];
      M = mm[&i];
	  t = &&ts&i;
	  output;
   %end;

run;

proc sort data=repdat0;
	by btsamp idb t;
run;

ODS SELECT NONE;

/**** M model - will use results for Y model in 2 step approach ***/

proc nlmixed data=repdat0;
   parms g0=0 g1=0 g2=0 g3=0 g4=0 km=1 nu2=1 sm2=1;

   lmum=g0 + (g1*X) + (g2*C1) + (g3*C2) - log((1 + exp(-g4*t))**(1/nu2)) + um;
   mum=exp(lmum);
   pm = 1 / (1 + mum * km);
   nm = 1 / km;
   model M ~ negbin(nm,pm);
   random um ~ normal(0,sm2) subject=idb;		* note that sm2 is the variance;

   predict um out=raneffm;
   predict lmum out=nlpredm;
   by btsamp;

   ods output ParameterEstimates = nlestsM;
   ods output ConvergenceStatus = convM;
run;

proc sort data=nlpredm;
  by btsamp id t;
run;

data predm (keep = btsamp idb t X lmum);
  set nlpredm;
  lmum = pred;
run;

proc sort data = predm;
  by btsamp idb t;
run;

data repdatYM;
  merge repdat0 predm;
  by btsamp idb t;
run;

 /**** Y Model ****/

proc nlmixed data=repdatYM;
  parms b0=0 b1=0 b2=0 b3=0 b4=0 b5=0 b6=0 k=1 nu=1 sy2=1;

  lmuyp = b0 + (b1*X) + (b2*C1) + (b3*C2) 
		- log((1 + exp(-(b4*t + b5*lmum*t)))**(1/nu)) + (b6*lmum);
  lmuy = lmuyp + uy;
  muy=exp(lmuy);
  py=1 / (1 + muy * k);
  ny=1 / k;
  model Y ~ negbin(ny,py);
  random uy ~ normal(0,sy2) subject=idb;		* note that sy is the variance;

  predict uy out=raneffy;
  predict lmuyp out=raneffp;
  predict lmuy out=raneffpr;  
  by btsamp;

  ods output ParameterEstimates = nlestsY;
  ods output ConvergenceStatus = convY;
run;

/* get merged file for necessary screen of bootstrap samples */

data convM2 (keep = btsamp conM);
  set convM;
  conM = status;
run;

data convY2 (keep = btsamp conY);
  set convY;
  conY = status;
run;

proc sort data = convM2;
  by btsamp;
run;

proc sort data = convY2;
  by btsamp;
run;

data nlestsM2 (keep = btsamp sm2est);
  set nlestsM;
  if parameter = 'sm2';
  sm2est = estimate;  
run;

data nlestsY2 (keep = btsamp sy2est);
  set nlestsY;
  if parameter = 'sy2';
  sy2est = estimate;
run;

proc sort data = nlestsM2;
  by btsamp;
run;

proc sort data = nlestsY2;
  by btsamp;
run;

data convSD;
  merge convM2 convY2 nlestsM2 nlestsY2;
  by btsamp;
run;

proc iml;
	del = &delg;  
	t1 = &t1;
	mtim = &mtim;

* set default values for start point and end point in x axis for plot;     
%if &tgd1 = . %then %let tg1 = &ts1;        * default value is &ts1;             
%if &tgd1 ^= . %then %let tg1 = &tgd1;

%if &tgd2 = . %then %let tg2 = &&ts&nt;     * default value is &&ts&nt;  
%if &tgd2 ^= . %then %let tg2 = &tgd2;


	* Create data set containing bootstrap and orig estimates;

    create mdent var{rep ts LE11 LE01 LE1P LE0P LE10 LE00 E11 E01 E1P E0P E10 E00 
				     I1P I0P D1 I0 D0 I1 TOT PI0 PI1 PI1P PI0P PD0 PD1 mm1 mm0 mm1t
				     LE11A LE1PA LE0PA LE01A LE10A LE00A E11A E1PA E0PA E01A E10A E00A
                     D0A I0A I1A I1PA I0PA TOTA PI1PA PI0PA PI0A PI1A PD0A PD1A};

	use dat2;	

    read all var{C1 C2} into CMATO;

    CMAT = CMATO @ J(&mult,1,1);	*multipler of reference group;
    
	n=nrow(CMAT);

	use convSD;
    read all var{btsamp} where(conM = 0 & conY=0 & sm2est>0 & sy2est>0) into R;
	
	call randseed(&randseed);  

	do rep=1 to nrow(R);

        rp=R[rep,1];

		use nlestsM;
		read all var{Estimate} where(btsamp=rp) into G;

		use nlestsY;
		read all var{Estimate} where(btsamp=rp) into B;

	    use raneffy;
        read all var{C1 C2} where(btsamp=rp & t=&ts1) into WI;	  

		* Get covs for each boot;

		W = WI @ J(&mult,1,1);

		/***** Get estimates for mediation formula approach;
		* Get estimates of expected potential outcomes;	 */

		*M model;

		g0=G[1, 1];		  * Intercept (M model);
		g1=G[2, 1];		  * Coef of X (M model);
		g2=G[3, 1];		  * Coef of C1 (M model);
		g3=G[4, 1];		  * Coef of C2 (M model);
		g4=G[5, 1];		  * Coef of time (M model);
		km=G[6, 1];		  * Damping param (M model) for X = 0;
		nu2=G[7, 1];		  * dispersion parameter (neg bin) for M model;
		sduM = sqrt(G[8, 1]);	  * Stdev for rand eff for M model;  

		GW=G[3:4, 1];		  * Coefficients of covariates;
		
		*Y model;

		b0=B[1, 1];  * Intercept (Y model);
		b1=B[2, 1];  * Coef of X (Y model);
		b2=B[3, 1];		  * Coef of C1 (Y Model);
		b3=B[4, 1];		  * Coef of C2 (Y Model);
        b4=B[5, 1];		  * Coef of time (Y model);
		b5=B[6, 1];       * Coef of lmum x time interaction;
        b6=B[7, 1];  	  * Coef of log mean of M (Y model);
		k=B[8, 1];		  * dispersion parameter (negbin) for Y model;
		nu=B[9, 1];		  * damping param (Y model) for X=0;
		sduY = sqrt(B[10, 1]);	  * Stdev for rand eff for M model;  

        BW=B[3:4, 1];

		* Initialize for AUCs;
		LE11A=0;
		LE10A=0;
		LE01A=0;
		LE1PA=0; 
        LE0PA=0; 

		LE00A=0;
	
		E11A=0;
		E10A=0;
		E01A=0;
		E1PA=0; 
		E0PA=0; 

		E00A=0;

	    UM = J(n,1);             /* Initialize vector of random effects for M */
	    UY = J(n,1);             /* Initialize vector of random effects for M */

		call randgen(UM, "Normal", 0, sduM);   /* the last item is the standard deviation  */
		call randgen(UY, "Normal", 0, sduY);   /* random draws for random effects */

		do ts = &tg1 to &tg2 by del; 
			E11T=0;
			E10T=0;

			E1PT=0; 
			E0PT=0; 

			E01T=0;

			E00T=0;

			* constant over i (people) and mi level of M;
			denm = (1/nu2)*log(1 + exp(-g4 * ts));

			denm1 = (1/nu2)*log(1 + exp(-g4 * t1));  

			if ts <= t1 then indt1 = 0; else indt1 = 1; 

			do i=1 to n;
				* Will sum over empir dist for all individuals;
				cov = W[i, ];
				umi = UM[i,];
				uyi = UY[i,];

				gc = cov * GW;
				bc = cov * BW;
	
				lmm1 = g0 + g1 + gc + umi - denm;
				lmm0 = g0 + gc + umi - denm;

				lmm1t1 = g0 + g1 + gc + umi - denm1;  * at t=t1;
				lmm0t1 = g0 + gc + umi - denm1;       * at t=t1;

				mm1 = exp(lmm1);
				mm0 = exp(lmm0);
				mm1t1 = exp(lmm1t1);			
				mm0t1 = exp(lmm0t1);			

				/* get mean for M at time t for intervention A1 */

				/* add outer IF statement to allow t1 = 0 */

				if mtim = 1 then do;
				  if t1 = 0 then mm1t = mm0; else do;
					if indt1 = 0 then mm1t = mm1;
					  else mm1t = mm1t1 - mm0t1 + mm0;
				    end;
				  end;
				else do;
				  if t1 = 0 then mm1t = mm1; else do;
					if indt1 = 0 then mm1t = mm0;
					  else mm1t = mm0t1 - mm1t1 + mm1;
				  end;
				end;

				lmm1t = log(mm1t);

				* expec M for person x=0;

				dlin1 = -b4 * ts - b5 * ts * lmm1;
				dlin0 = -b4 * ts - b5 * ts * lmm0;
				
				dlinp = -b4 * ts - b5 * ts * lmm1t; 

				* Truncate these linear predictors if too big;

				dlint = 10;
				maxv = 80;

				if dlin1 < 0 then signd1 = -1; else signd1 = 1;

				if abs(dlin1) > dlint then 
					dlin1 = signd1 * dlint;  

				if dlin0 < 0 then signd0 = -1; else signd0 = 1;

				if abs(dlin0) > dlint then 
					dlin0 = signd0 * dlint;
 
				if dlinp < 0 then signdp = -1; else signdp = 1;

				if abs(dlinp) > dlint then 
					dlinp = signdp * dlint;  

				dlinm = max(dlin1,dlin0,dlinp);			
				numin = log(1 + exp(dlinm)) / (maxv*log(10));  /* set limit for nu */

				/* The above implies a max value for deny1 = 10**maxv */

				if nu < numin then nu = numin;  

				deny1 = (1 + exp(dlin1))**(1/nu); 
				deny0 = (1 + exp(dlin0))**(1/nu); 

				denyp = (1 + exp(dlinp))**(1/nu);  

				EL1 = b0 + (b6*lmm1) + bc + uyi;
				EL0 = b0 + (b6*lmm0) + bc + uyi;
				ELP = b0 + (b6*lmm1t) + bc + uyi;

      		    E11i = exp(EL1 + b1) / deny1;
				E01i = exp(EL1) / deny1;       
				E10i = exp(EL0 + b1) / deny0;

				E1Pi = exp(ELP + b1) / denyp;		
				E0Pi = exp(ELP + b1) / denyp;	

				E00i = exp(EL0) / deny0;

  			    E11T = E11T + E11i;
				E01T = E01T + E01i;		
				E10T = E10T + E10i;	

				E1PT = E1PT + E1Pi;		
				E0PT = E0PT + E0Pi;		
				E00T = E00T + E00i;

			end;
			
			* sum over people;
						
			   E11 = E11T / n;
			   E01 = E01T / n;
			   E10 = E10T / n;
			   E1P = E1PT / n;   
			   E0P = E0PT / n;   

			   E00 = E00T / n;

			LE11=log(E11);
			LE01=log(E01);
			LE10=log(E10);
			LE1P=log(E1P);		
			LE0P=log(E0P);		

			LE00=log(E00);

         if &scale = 1 then do;     * scale=1 for ratio scale;

			D1 = LE11 - LE01;
			I0 = LE01 - LE00;
			D0 = LE10 - LE00;
			I1 = LE11 - LE10;
			I1P = LE11 - LE1P;  
			I0P = LE01 - LE0P;  

			TOT = D0 + I1;
			PI0 = I0 / TOT;    
			PI1 = I1 / TOT;
			PI1P = I1P / TOT;	
            PI0P = I0P / TOT;	

			PD0 = D0 / TOT;   
			PD1 = D1 / TOT;

	     end;

	     if &scale = 2 then do;      * scale=2 for difference scale;

			D1 = E11 - E01;   
			I0 = E01 - E00;
			D0 = E10 - E00;
			I1 = E11 - E10;
			I1P = E11 - E1P;
			I0P = E01 - E0P;  

			TOT = D0 + I1;
			PI0 = I0 / TOT;    
			PI1 = I1 / TOT;
			PI1P = I1P / TOT;	
			PI0P = I0P / TOT;	

			PD0 = D0 / TOT;    
			PD1 = D1 / TOT;

		  end;

			if (ts=&tg1 | ts=&tg2) then
				div=2;
			else
				div=1;

         if &scale = 1 then do;     
			LE11A = LE11A + (LE11 / div);
			LE01A = LE01A + (LE01 / div);
			LE10A = LE10A + (LE10 / div);
			LE1PA = LE1PA + (LE1P / div);  
			LE0PA = LE0PA + (LE0P / div);  
			LE00A = LE00A + (LE00 / div);

			D1A = del * (LE11A - LE01A);
			I0A = del * (LE01A - LE00A);	
			D0A = del * (LE10A - LE00A);
			I1A = del * (LE11A - LE10A);
			I1PA = del * (LE11A - LE1PA);  
			I0PA = del * (LE01A - LE0PA);  

			TOTA = del * (LE11A - LE00A);

	     end;

	     if &scale = 2 then do;      

		    E11A = E11A + (E11 / div);
			E01A = E01A + (E01 / div);  
			E10A = E10A + (E10 / div);
			E1PA = E1PA + (E1P / div);  
			E0PA = E0PA + (E0P / div);  
     		E00A = E00A + (E00 / div);

			D1A = del * (E11A - E01A);
			I0A = del * (E01A - E00A);  
			D0A = del * (E10A - E00A);
			I1A = del * (E11A - E10A);
			I1PA = del * (E11A - E1PA);  
			I0PA = del * (E01A - E0PA);  

			TOTA = del * (E11A - E00A);

		  end;

			PI1A = I1A / TOTA;
			PI0A = I0A / TOTA;		
			PI1PA = I1PA / TOTA;	
			PI0PA = I0PA / TOTA;	

			PD0A = D0A / TOTA;  
			PD1A = D1A / TOTA;

			append var{rep ts LE11 LE01 LE1P LE0P LE10 LE00 E11 E01 E1P E0P E10 E00 
				       I1P I0P D1 I0 D0 I1 TOT PI0 PI1 PI1P PI0P PD0 PD1 mm1 mm0 mm1t
				       LE11A LE1PA LE0PA LE01A LE10A LE00A E11A E1PA E0PA E01A E10A E00A
                       D0A I0A I1A I1PA I0PA TOTA PI1PA PI0PA PI0A PI1A PD0A PD1A};

			* append to mdent;
		end;
		* do over time index ti;
	end;
	* replicates (bootstrap and original);

ODS SELECT ALL;   /* Allow printing to output at this point */

data mdentb;      * Results from bootstrap reps;
	set mdent;
	if rep gt 1 then boot=1;
	else boot=0;
	if boot=1;
run;

data mdento;     * Results from original dataset;
	set mdent;
	if rep gt 1 then
		boot=1;
	else
		boot=0;
	if boot=0;
	D1O=D1;		
	I0O=I0;
	TOTO=TOT;
	PI0O=PI0;	
	D0O=D0;
	I1O=I1;
	I1PO = I1P;
	I0PO = I0P;   
	PI1O = PI1;
	PI0O = PI0;
	PI1PO = PI1P;	
	PI0PO = PI0P;	
	PD0O = PD0;  
	PD1O = PD1;
	mm1o=mm1;
	mm0o=mm0;
	mm1to = mm1t;

	keep ts LE11 LE01 LE1P LE0P LE10 LE00 E11 E1P E0P E10 E01 E00
         I1PO I0PO D1O I0O TOTO PI0O PI1PO PI0PO D0O I1O PI1O PD0O PD1O mm1o mm0o mm1to
		 LE11A LE1PA LE0PA LE01A LE10A LE00A E11A E1PA E0PA E10A E01A E00A
         D0A I1A I0A I1PA I0PA TOTA PI0A PI1PA PI0PA PI1A PD0A PD1A;
run;

proc sort data=mdento;
	by ts;
run;

* Note that the ABC - area between the curves - is at the last time;

proc sort data=mdentb;
	by ts;
run;

proc univariate data=mdentb noprint;
	var D0 D1 I1 I0 I1P I0P TOT PI1 PI0 PI1P PI0P PD0 PD1 D0A I1A I0A I1PA I0PA TOTA PI1A PI0A PI1PA PI0PA PD0A PD1A;
	output out=CIBNDS pctlpts=2.5 97.5 pctlpre=D0_ D1_ I1_ I0_ I1P_ I0P_ TOT_ PI1_ PI0_ PI1P_ PI0P_ PD0_ PD1_ D0A_ I1A_ I0A_ I1PA_ I0PA_ TOTA_ 
		PI1A_ PI0A_ PI1PA_ PI0PA_ PD0A_ PD1A_ pctlname=LB95 UB95 stderr=seD0 seD1 seI1 seI0 seI1P seI0P seTOT sePI1 sePI0 sePI1P sePI0P sePD0 sePD1 seD0A seI1A seI0A 
		seI1PA seI0PA seTOTA sePI1A sePI0A sePI1PA sePI0PA sePD0A sePD1A n=ND0 ND1 NI1 NI0 NI1P NI0P NTOT NPI1 NPI0 NPI1P NPI0P NPD0 NPD1 ND0A NI1A NI0A NI1PA NI0PA 
		NTOTA NPI1A NPI0A NPI1PA NPI0PA NPD0A NPD1A;
	by ts;
run;

data xmdento;
  set mdento;
  E11 = exp(LE11);
  E01 = exp(LE01);
  E10 = exp(LE10);
  E1P = exp(LE1P);
  E0P = exp(LE0P); 
  E00 = exp(LE00);
  XD1O = exp(D1O);
  XI0O = exp(I0O);
  XTOTO = exp(TOTO);
  XD0O = exp(D0O);
  XI1O = exp(I1O);
  XI1PO = exp(I1PO);
  XI0PO = exp(I0PO); 
run;

data XCIBNDS;
  set CIBNDS;
  XD0_LB95 = exp(D0_LB95);
  XD0_UB95 = exp(D0_UB95);
  XD1_LB95 = exp(D1_LB95);      
  XD1_UB95 = exp(D1_UB95);
  XI1_LB95 = exp(I1_LB95);
  XI1_UB95 = exp(I1_UB95);
  XI0_LB95 = exp(I0_LB95);
  XI0_UB95 = exp(I0_UB95);
  XI1P_LB95 = exp(I1P_LB95);	
  XI1P_UB95 = exp(I1P_UB95);
  XI0P_LB95 = exp(I0P_LB95);	
  XI0P_UB95 = exp(I0P_UB95);    
  XTOT_LB95 = exp(TOT_LB95);
  XTOT_UB95 = exp(TOT_UB95);
run;

/************************** Figure (same for ratio and diff scale)*******************/

/* Create the Annotate data set LINETEXT */
data linetext;   
   length function $8 text $20;        
  retain function 'label' xsys ysys '2' hsys '3' position '6' size 2.5;                                 
 								
   set mdento end=last;
   if last then do;
      function='label'; text=' A^1'; position='6'; x=ts; y=E11; output;
      function='label'; text=' A^(1,t1)'; position='6'; x=ts; y=E1P; output;
      function='label'; text=' A^(1,0)'; position='6'; x=ts; y=E10; output;
      function='label'; text=' A^0'; position='6'; x=ts; y=E00; output;
   end;
run;
              
* set default values for start point and end point in x axis for plot;     
%if &xstart = . %then %let xst = &ts1;                 * default value is &ts1;             
%if &xstart ^= . %then %let xst = &xstart;

%if &xend = . %then %let xen = %eval(&&ts&nt + 10);    * default value is (&&ts&nt + 10);  
%if &xend ^= . %then %let xen = &xend;


title 'Predicted Mean by Time';
proc gplot data=mdento;
   symbol1 i=join color=red l=1 width = 2;   
   symbol2 i=join color=brown l=5 width = 2;
   symbol3 i=join color=green l=8 width = 2;
   symbol4 i=join color=blue l=34 width = 2;
  
   axis1 label=(height=1.5 "time") 
         value=(height=1.5)
         order=(&xst to &xen by &interval);
   axis2 label=(angle=90 height=1.5 "Pred Mean" ) 
         value=(height=1.5);
   plot (E11 E1P E10 E00)* ts/overlay haxis=axis1 vaxis=axis2 annotate=linetext;
Run; * graph for E11 E1P E10 E00;



 %if &scale = 1 %then %do;     

/************************************* table: Direct effect *************************************/
data dest;
	set xmdento;
	where ts = &&ts&nt;
	keep XD0O;
run;

data cilower;
	set XCIBNDS;
	where ts = &&ts&nt;
	keep XD0_LB95;
run;

data ciupper;
	set XCIBNDS;
	where ts = &&ts&nt;
	keep XD0_UB95;
run;

data propdest;
	set xmdento;
	where ts = &&ts&nt;
	keep PD0O;
run;

data propcilower;
	set XCIBNDS;
	where ts = &&ts&nt;
	keep PD0_LB95;
run;

data propciupper;
	set XCIBNDS;
	where ts = &&ts&nt;
	keep PD0_UB95;
run;

proc datasets nolist;
	modify dest;
		rename XD0O=D_&&ts&nt;
	modify cilower;
		rename XD0_LB95=D_&&ts&nt;
	modify ciupper;
		rename XD0_UB95=D_&&ts&nt;
	modify propdest;
		rename PD0O=D_&&ts&nt;
	modify propcilower;
		rename PD0_LB95=D_&&ts&nt;
	modify propciupper;
		rename PD0_UB95=D_&&ts&nt;
run;

data temp1;
	retain D_&&ts&nt;
	set dest cilower ciupper propdest propcilower propciupper;
	keep D_&&ts&nt;
run;

proc transpose data = temp1 out = temp2;
run;

data temp2;
	set temp2;
	keep newname _NAME_ COL1 COL2 COL3 COL4 COL5 COL6;
	newname = _NAME_;
run;

data forprint;
	retain newname COL1 COL2 COL3 COL4 COL5 COL6 _NAME_;
	set temp2;
	keep newname COL1 COL2 COL3 COL4 COL5 COL6;	
	label newname = 'Effect';
	label COL1 = 'Estimate';
	label COL2 = '95% CI Lower';
	label COL3 = '95% CI Upper';
	label COL4 = 'Prop.';
	label COL5 = '95% CI Lower';
	label COL6 = '95% CI Upper';
run;

/******************************** table: Indirect effect  ********************************/
%do i=1 %to &na;
   data inest&i;
   set xmdento;
   where ts = &&ta&i;
   keep XI1O;
%end;

%do i=1 %to &na;
   data incilower&i;
   set XCIBNDS;
   where ts = &&ta&i;
   keep XI1_LB95;
%end;
   
%do i=1 %to &na;
   data inciupper&i;
   set XCIBNDS;
   where ts = &&ta&i;
   keep XI1_UB95;
%end;

%do i=1 %to &na;
   data inpropest&i;
   set xmdento;
   where ts = &&ta&i;
   keep PI1O;
%end;

%do i=1 %to &na;
   data inpropcilower&i;
   set XCIBNDS;
   where ts = &&ta&i;
   keep PI1_LB95;
%end;
   
%do i=1 %to &na;
   data inpropciupper&i;
   set XCIBNDS;
   where ts = &&ta&i;
   keep PI1_UB95;
%end;

proc datasets nolist;
%do i=1 %to &na;
	modify inest&i;
		rename XI1O=I_&&ta&i;
	modify incilower&i;
		rename XI1_LB95=I_&&ta&i;
	modify inciupper&i;
		rename XI1_UB95=I_&&ta&i;

	modify inpropest&i;
		rename PI1O=I_&&ta&i;
	modify inpropcilower&i;
		rename PI1_LB95=I_&&ta&i;
	modify inpropciupper&i;
		rename PI1_UB95=I_&&ta&i;
%end;
run;

%do i=1 %to &na;
data intemp&i;
	retain I_&&ta&i;
	set inest&i incilower&i inciupper&i inpropest&i inpropcilower&i inpropciupper&i;
	keep I_&&ta&i;
run;
%end;

%do i=1 %to &na;
proc transpose data = intemp&i out = intemp2&i;
run;
%end;

data intempall;
set
%do i = 1 %to &na;
  intemp2&i
%end;
;
run;

data intempall;
	set intempall;
	keep newname _NAME_ COL1 COL2 COL3 COL4 COL5 COL6;
	newname = _NAME_;
run;

data inforprint;
	retain newname COL1 COL2 COL3 COL4 COL5 COL6 _NAME_;
	set intempall;
	keep newname COL1 COL2 COL3 COL4 COL5 COL6;	
	label newname = 'Effect';
	label COL1 = 'Estimate';
	label COL2 = '95% CI Lower';
	label COL3 = '95% CI Upper';
	label COL4 = 'Prop.';
	label COL5 = '95% CI Lower';
	label COL6 = '95% CI Upper';
run;

* ABC(Prop);
data inaest;
   set xmdento;
   where ts = &&ts&nt;
   keep I1A;
run;

data inacilower;
   set XCIBNDS;
   where ts = &&ts&nt;
   keep I1A_LB95;
run;
   
data inaciupper;
   set XCIBNDS;
   where ts = &&ts&nt;
   keep I1A_UB95;
run;

data inpropaest;
   set xmdento;
   where ts = &&ts&nt;
   keep PI1A;
run;

data inpropacilower;
   set XCIBNDS;
   where ts = &&ts&nt;
   keep PI1A_LB95;
run;
   
data inpropaciupper;
   set XCIBNDS;
   where ts = &&ts&nt;
   keep PI1A_UB95;
run;

proc datasets nolist;
	modify inaest;
		rename I1A=ABC;
	modify inacilower;
		rename I1A_LB95=ABC;
	modify inaciupper;
		rename I1A_UB95=ABC;

	modify inpropaest;
		rename PI1A=ABC;
	modify inpropacilower;
		rename PI1A_LB95=ABC;
	modify inpropaciupper;
		rename PI1A_UB95=ABC;
run;

data intempa;
	retain ABC;
	set inaest inacilower inaciupper inpropaest inpropacilower inpropaciupper;
	keep ABC;
run;

proc transpose data = intempa out = intempa2;
run;

data intempa2;
	set intempa2;
	keep newname _NAME_ COL1 COL2 COL3 COL4 COL5 COL6;
	newname = _NAME_;
run;

data inaforprint;
	retain newname COL1 COL2 COL3 COL4 COL5 COL6 _NAME_;
	set intempa2;
	keep newname COL1 COL2 COL3 COL4 COL5 COL6;		
	label newname = 'Effect';
	label COL1 = 'Estimate';
	label COL2 = '95% CI Lower';
	label COL3 = '95% CI Upper';
	label COL4 = 'Prop.';
	label COL5 = '95% CI Lower';
	label COL6 = '95% CI Upper';
run;

* ABC(Prop) - partial mediation;

data inpaest;
   set xmdento;
   where ts = &&ts&nt;
   keep I1PA;
run;

data inpacilower;
   set XCIBNDS;
   where ts = &&ts&nt;
   keep I1PA_LB95;
run;

data inpaciupper;
   set XCIBNDS;
   where ts = &&ts&nt;
   keep I1PA_UB95;
run;

data inppropaest;
   set xmdento;
   where ts = &&ts&nt;
   keep PI1PA;
run;

data inppropacilower;
   set XCIBNDS;
   where ts = &&ts&nt;
   keep PI1PA_LB95;
run;

data inppropaciupper;
   set XCIBNDS;
   where ts = &&ts&nt;
   keep PI1PA_UB95;
run;

proc datasets nolist;
	modify inpaest;
		rename I1PA=ABC_&t1;
	modify inpacilower;
		rename I1PA_LB95=ABC_&t1;
	modify inpaciupper;
		rename I1PA_UB95=ABC_&t1;

	modify inppropaest;
		rename PI1PA=ABC_&t1;
	modify inppropacilower;
		rename PI1PA_LB95=ABC_&t1;
	modify inppropaciupper;
		rename PI1PA_UB95=ABC_&t1;
run;

data inptempa;
	retain ABC_&t1;
	set inpaest inpacilower inpaciupper inppropaest inppropacilower inppropaciupper;
	keep ABC_&t1;
run;

proc transpose data = inptempa out = inptempa2;
run;

data inptempa2;
	set inptempa2;
	keep newname _NAME_ COL1 COL2 COL3 COL4 COL5 COL6;
	newname = _NAME_;
run;

data inpa;
	retain newname COL1 COL2 COL3 COL4 COL5 COL6 _NAME_;
	set inptempa2;
	keep newname COL1 COL2 COL3 COL4 COL5 COL6;	
	label newname = 'Effect';
	label COL1 = 'Estimate';
	label COL2 = '95% CI Lower';
	label COL3 = '95% CI Upper';
	label COL4 = 'Prop.';
	label COL5 = '95% CI Lower';
	label COL6 = '95% CI Upper';
run;

data final;
 set forprint inforprint inaforprint inpa;
run;

proc print data = final label;
	title 'Mediation effects, estimates and proportions - Ratio scale';
run;
%end;

%if &scale = 2  %then %do;     

/************************************* table: Direct effect *************************************/
%do i=1 %to &na;
   data dest&i;
   set mdento;
   where ts = &&ta&i;
   keep D0O D1O;
%end;

%do i=1 %to &na;
   data cilower&i;
   set CIBNDS;
   where ts = &&ta&i;
   keep D0_LB95 D1_LB95;
%end;
   
%do i=1 %to &na;
   data ciupper&i;
   set CIBNDS;
   where ts = &&ta&i;
   keep D0_UB95 D1_UB95;
%end;

%do i=1 %to &na;
   data propest&i;
   set mdento;
   where ts = &&ta&i;
   keep PD0O PD1O;
%end;

%do i=1 %to &na;
   data propcilower&i;
   set CIBNDS;
   where ts = &&ta&i;
   keep PD0_LB95 PD1_LB95;
%end;
   
%do i=1 %to &na;
   data propciupper&i;
   set CIBNDS;
   where ts = &&ta&i;
   keep PD0_UB95 PD1_UB95;
%end;

proc datasets nolist;
%do i=1 %to &na;
	modify dest&i;
		rename D0O=D0_&&ta&i D1O=D1_&&ta&i;
	modify cilower&i;
		rename D0_LB95=D0_&&ta&i D1_LB95=D1_&&ta&i;
	modify ciupper&i;
		rename D0_UB95=D0_&&ta&i D1_UB95=D1_&&ta&i;
	modify propest&i;
		rename PD0O=D0_&&ta&i PD1O=D1_&&ta&i;
	modify propcilower&i;
		rename PD0_LB95=D0_&&ta&i PD1_LB95=D1_&&ta&i;
	modify propciupper&i;
		rename PD0_UB95=D0_&&ta&i PD1_UB95=D1_&&ta&i;
%end;
run;

%do i=1 %to &na;
data temp&i;
	retain D0_&&ta&i D1_&&ta&i;
	set dest&i cilower&i ciupper&i propest&i propcilower&i propciupper&i;
	keep D0_&&ta&i D1_&&ta&i;
run;
%end;

%do i=1 %to &na;
proc transpose data = temp&i out = temp2&i;
run;
%end;

data tempall;
set
%do i = 1 %to &na;
  temp2&i
%end;
;
run;

data tempall;
	set tempall;
	keep newname _NAME_ COL1 COL2 COL3 COL4 COL5 COL6;
	newname = _NAME_;
run;

data forprint;
	retain newname COL1 COL2 COL3 COL4 COL5 COL6 _NAME_;
	set tempall;
	keep newname COL1 COL2 COL3 COL4 COL5 COL6;	
	label newname = 'Effect';
	label COL1 = 'Estimate';
	label COL2 = '95% CI Lower';
	label COL3 = '95% CI Upper';
	label COL4 = 'Prop.';
	label COL5 = '95% CI Lower';
	label COL6 = '95% CI Upper';
run;

/******************************** table: Indirect effect ********************************/
%do i=1 %to &na;
   data inest&i;
   set mdento;
   where ts = &&ta&i;
   keep I1O I0O;
%end;

%do i=1 %to &na;
   data incilower&i;
   set CIBNDS;
   where ts = &&ta&i;
   keep I1_LB95 I0_LB95;
%end;
   
%do i=1 %to &na;
   data inciupper&i;
   set CIBNDS;
   where ts = &&ta&i;
   keep I1_UB95 I0_UB95;
%end;

%do i=1 %to &na;
   data inpropest&i;
   set mdento;
   where ts = &&ta&i;
   keep PI1O PI0O;
%end;

%do i=1 %to &na;
   data inpropcilower&i;
   set CIBNDS;
   where ts = &&ta&i;
   keep PI1_LB95 PI0_LB95;
%end;
   
%do i=1 %to &na;
   data inpropciupper&i;
   set CIBNDS;
   where ts = &&ta&i;
   keep PI1_UB95 PI0_UB95;
%end;

proc datasets nolist;
%do i=1 %to &na;
	modify inest&i;
		rename I1O=I1_&&ta&i I0O=I0_&&ta&i;
	modify incilower&i;
		rename I1_LB95=I1_&&ta&i I0_LB95=I0_&&ta&i;
	modify inciupper&i;
		rename I1_UB95=I1_&&ta&i I0_UB95=I0_&&ta&i;

	modify inpropest&i;
		rename PI1O=I1_&&ta&i PI0O=I0_&&ta&i;
	modify inpropcilower&i;
		rename PI1_LB95=I1_&&ta&i PI0_LB95=I0_&&ta&i;
	modify inpropciupper&i;
		rename PI1_UB95=I1_&&ta&i PI0_UB95=I0_&&ta&i;
%end;
run;

%do i=1 %to &na;
data intemp&i;
	retain I1_&&ta&i I0_&&ta&i;
	set inest&i incilower&i inciupper&i inpropest&i inpropcilower&i inpropciupper&i;
	keep I1_&&ta&i I0_&&ta&i;
run;
%end;

%do i=1 %to &na;
proc transpose data = intemp&i out = intemp2&i;
run;
%end;

data intempall;
set
%do i = 1 %to &na;
  intemp2&i
%end;
;
run;

data intempall;
	set intempall;
	keep newname _NAME_ COL1 COL2 COL3 COL4 COL5 COL6;
	newname = _NAME_;
run;

data inforprint;
	retain newname COL1 COL2 COL3 COL4 COL5 COL6 _NAME_;
	set intempall;
	keep newname COL1 COL2 COL3 COL4 COL5 COL6;	
	label newname = 'Effect';
	label COL1 = 'Estimate';
	label COL2 = '95% CI Lower';
	label COL3 = '95% CI Upper';
	label COL4 = 'Prop.';
	label COL5 = '95% CI Lower';
	label COL6 = '95% CI Upper';
run;

* ABC(Prop);

data inaest;
   set mdento;
   where ts = &&ts&nt;
   keep I1A I0A;
run;

data inacilower;
   set CIBNDS;
   where ts = &&ts&nt;
   keep I1A_LB95 I0A_LB95;
run;

data inaciupper;
   set CIBNDS;
   where ts = &&ts&nt;
   keep I1A_UB95 I0A_UB95;
run;

data inpropaest;
   set mdento;
   where ts = &&ts&nt;
   keep PI1A PI0A;
run;

data inpropacilower;
   set CIBNDS;
   where ts = &&ts&nt;
   keep PI1A_LB95 PI0A_LB95;
run;

data inpropaciupper;
   set CIBNDS;
   where ts = &&ts&nt;
   keep PI1A_UB95 PI0A_UB95;
run;

proc datasets nolist;
	modify inaest;
		rename I1A=ABC1 I0A=ABC0;
	modify inacilower;
		rename I1A_LB95=ABC1 I0A_LB95=ABC0;
	modify inaciupper;
		rename I1A_UB95=ABC1 I0A_UB95=ABC0;

	modify inpropaest;
		rename PI1A=ABC1 PI0A=ABC0;
	modify inpropacilower;
		rename PI1A_LB95=ABC1 PI0A_LB95=ABC0;
	modify inpropaciupper;
		rename PI1A_UB95=ABC1 PI0A_UB95=ABC0;
run;

data intempa;
	retain ABC1 ABC0;
	set inaest inacilower inaciupper inpropaest inpropacilower inpropaciupper;
	keep ABC1 ABC0;
run;

proc transpose data = intempa out = intempa2;
run;

data intempa2;
	set intempa2;
	keep newname _NAME_ COL1 COL2 COL3 COL4 COL5 COL6;
	newname = _NAME_;
run;

data inaforprint;
	retain newname COL1 COL2 COL3 COL4 COL5 COL6 _NAME_;
	set intempa2;
	keep newname COL1 COL2 COL3 COL4 COL5 COL6;		
	label newname = 'Effect';
	label COL1 = 'Estimate';
	label COL2 = '95% CI Lower';
	label COL3 = '95% CI Upper';
	label COL4 = 'Prop.';
	label COL5 = '95% CI Lower';
	label COL6 = '95% CI Upper';
run;

* ABC(Prop) - partial mediation;

data inpaest;
   set mdento;
   where ts = &&ts&nt;
   keep I1PA I0PA;
run;

data inpacilower;
   set CIBNDS;
   where ts = &&ts&nt;
   keep I1PA_LB95 I0PA_LB95;
run;

data inpaciupper;
   set CIBNDS;
   where ts = &&ts&nt;
   keep I1PA_UB95 I0PA_UB95;
run;

data inppropaest;
   set mdento;
   where ts = &&ts&nt;
   keep PI1PA PI0PA;
run;

data inppropacilower;
   set CIBNDS;
   where ts = &&ts&nt;
   keep PI1PA_LB95 PI0PA_LB95;
run;

data inppropaciupper;
   set CIBNDS;
   where ts = &&ts&nt;
   keep PI1PA_UB95 PI0PA_UB95;
run;

proc datasets nolist;
	modify inpaest;
		rename I1PA=ABC1_&t1 I0PA=ABC0_&t1;
	modify inpacilower;
		rename I1PA_LB95=ABC1_&t1 I0PA_LB95=ABC0_&t1;
	modify inpaciupper;
		rename I1PA_UB95=ABC1_&t1 I0PA_UB95=ABC0_&t1;

	modify inppropaest;
		rename PI1PA=ABC1_&t1 PI0PA=ABC0_&t1;
	modify inppropacilower;
		rename PI1PA_LB95=ABC1_&t1 PI0PA_LB95=ABC0_&t1;
	modify inppropaciupper;
		rename PI1PA_UB95=ABC1_&t1 PI0PA_UB95=ABC0_&t1;
run;

data inptempa;
	retain ABC1_&t1 ABC0_&t1;
	set inpaest inpacilower inpaciupper inppropaest inppropacilower inppropaciupper;
	keep ABC1_&t1 ABC0_&t1;
run;

proc transpose data = inptempa out = inptempa2;
run;

data inptempa2;
	set inptempa2;
	keep newname _NAME_ COL1 COL2 COL3 COL4 COL5 COL6;
	newname = _NAME_;
run;

data inpa;
	retain newname COL1 COL2 COL3 COL4 COL5 COL6 _NAME_;
	set inptempa2;
	keep newname COL1 COL2 COL3 COL4 COL5 COL6;	
	label newname = 'Effect';
	label COL1 = 'Estimate';
	label COL2 = '95% CI Lower';
	label COL3 = '95% CI Upper';
	label COL4 = 'Prop.';
	label COL5 = '95% CI Lower';
	label COL6 = '95% CI Upper';
run;

data final;
 set forprint inforprint inaforprint inpa;
run;

proc print data = final label;
	title 'Mediation effects, estimates and proportions - Difference scale';
run;

%end;

%mend CMED;

/****************  End of Macro CMED *********************/

/**********  Example: CMED inputs for dental data  ******/

%CMED (499, 752910, 623218,
       dmft, 3, dmft8, dmft18, dmft36, , , 8, 18, 36, , ,
       sdrk, sdrk8, sdrk18, sdrk36, , ,
       ses, 
       bthgrp sex, 2, bthgrp, sex, , , ,
       ., ., 18, 1, 0.5, 10,
       ., ., 10,
       3, 8, 18, 36, , ,
       2, ECCM);
run;

/* Note that in the dental data analysis we used preliminary estimates to get starting values in Proc NLMIXED as follows:
	M model:  parms g0=0.2 g1=0.8 g2=0.2 g3=-0.1 g4=0.2 km=0.4 nu2=0.2 sm2=0.6;
    Y model:  parms b0=-2 b1=1 b2=0.2 b3=-0.2 b4=0.1 b5=0.1 b6=0.1 k=1 nu=0.1 sy2=4; 
*/

/********** Guide for CMED macro inputs  ***********/

/*%macro CMED (nboot, seed, randseed,
               yvar, nt, yt1, yt2, yt3, yt4, yt5, ts1, ts2, ts3, ts4, ts5,
               mvar, mt1, mt2, mt3, mt4, mt5,
               xvar, 
               cvar, nc, cvar1, cvar2, cvar3, cvar4, cvar5,
               tgd1, tgd2, t1, mtim, delg, mult,
               xstart, xend, interval,
               na, ta1, ta2, ta3, ta4, ta5,
               scale, datset);                 */
  

/*  Inputs for Macro CMED:
1) nboot = number of bootstrap samples,
2) seed = seed for bootstrap sampling,
3) ydar = name of final outcome variable (Y),
4) nt = number of time points,
5) yt1, yt2, ..., yt5 = values of outcome variable for each time point,
6) ts1, ts2, ..., ts5 = measurement times for each time point,
7) mvar = mediator variable (M),
8) mt1, mt2, ..., mt5 = values of mediator variable for each time point,
9) xvar = name of exposure or treatment variable (X),
10) cvar = list of (up to 5) covariates,
11) nc = number of covariates,
12) cvar1, cvar2, ..., cvar5 = list of (up to 5) covariates,(rewritten from cvar, include blanks if <5),
13) tgd1 = first measurement time,
14) tgd2 = last measurement time,
15) t1 = specifies a changepoint in an intervention,
16) mtim = indicates whether the partial mediation 
          starts at t1 (mtim = 1 - delayed intervention) or 
		  ends at t1 (mtim = 0 - curtailed intervention),
17) delg = delta for plot of predicted values,
18) mult = reference group (sample) multiplier for mediation formula,
19) xstart, xend = start point and end point in x axis in plot,
20) interval = interval in x axis in plot,
21) na =  number of time points that will output in the table,
22) ta1, ta2, ..., ta5 = measurement times for each time point that output in the table,
23) scale = specify scale used for mediation estimands:
            1 - mean ratio scale; 2 - difference scale,
24) datset = name of SAS datafile
*/
